#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <wchar.h>
#include <locale.h>
#include <string.h>
#include <wctype.h>

/*

input:

foo bar
foo

output:
      1 bar
      2 foo

*/

size_t hash(wchar_t* word) {
  char* s=(char*)word;
  size_t x;
  ssize_t i;
  /* looping backwards is more efficient */
  for (x=i=wcslen(word)*sizeof(wchar_t); i>=0; i--) {
    x += ~(s[i+1]) + s[i];
  }
//  fprintf(stderr,"%ls -> %c\n",word,x+1);
  return x;
}

#define HASHTABSIZE 65536

struct node {
  wchar_t* word;
  size_t count;
  struct node* next;
}* hashtab[HASHTABSIZE];

unsigned long words;

void* allocassert(void* x) {
  if (!x) {
    fprintf(stderr,"out of memory!\n");
    exit(1);
  }
  return x;
}

wchar_t* strdupw(wchar_t* word) {
  size_t len=(wcslen(word)+1)*sizeof(wchar_t);
  return realloc(word,len);
}

void countword(wchar_t* word) {
  struct node** x;
  size_t h=hash(word)%HASHTABSIZE;
  x=&hashtab[h];
  while (*x) {
    if (!wcscmp(word,(*x)->word)) {
      (*x)->count++;
      free(word);
      return;
    }
    x=&(*x)->next;
  }
  *x=allocassert(malloc(sizeof(struct node)));
  ++words;
  (*x)->next=0;
  (*x)->word=allocassert(strdupw(word));
  (*x)->count=1;
}

void readwords() {
  wchar_t* word;
  size_t i,wordlen;
  word=0; wordlen=0;
  for (i=0;;) {
    wint_t c=fgetwc(stdin);
    if (c==WEOF) break;
    if (i>=wordlen) {
      wordlen+=1024;
      word=allocassert(realloc(word,wordlen*sizeof(wchar_t)));
    }
    if (iswspace(c)) {
      if (i>0) {
	word[i]=0;
	countword(word);
	word=0; wordlen=0;
      }
      i=0;
      continue;
    }
    word[i++]=c;
  }
}

int compare(struct node* a, struct node* b) {
  long diff;
  return (diff=a->count-b->count)?diff:wcscmp(a->word,b->word);
}

void sortcounts() {
  size_t i,j;
  /* sort the chains */
  struct node* x;
  x=allocassert(malloc(words*sizeof(struct node)));
  for (i=j=0; i<HASHTABSIZE; i++) {
    while (hashtab[i]) {
      x[j++]=*hashtab[i];
      hashtab[i]=hashtab[i]->next;
    }
  }
  if (j!=words) {
    fprintf(stderr,"panic!\n");
    exit(1);
  }
  qsort(x,words,sizeof(x[0]),compare);
  for (i=0; i<words; i++)
    printf("%8zu %ls\n",x[i].count,x[i].word);
}

int main() {
  setlocale(LC_ALL,"");
  memset(hashtab,0,sizeof(hashtab));
  readwords();
  if (words) sortcounts();
  return 0;
}
